@@ -94,11 +94,11 @@ usage(char *prog)
 static u_int8_t
 count_dd(FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 {
-	if (flags & FS_FLAG_UNALLOC) 
+	if (flags & FS_FLAG_DATA_UNALLOC) 
 		uncnt++;
 
 	if (count-- == 0) {
-		if (flags & FS_FLAG_UNALLOC) 
+		if (flags & FS_FLAG_DATA_UNALLOC) 
 			printf("%d\n", uncnt);
 		else
 			printf("ERROR: unit is allocated, it will not be in an dls image\n");
@@ -143,7 +143,7 @@ main(int argc, char **argv)
 
 		case 'd':
 			type |= CALC_DD;
-			flags = FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_ALIGN | FS_FLAG_META;
+			flags = FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT;
 			count = atoi(optarg);
 			break;
 
@@ -153,7 +153,7 @@ main(int argc, char **argv)
 
 		case 'u':
 			type |= CALC_DLS;
-			flags = FS_FLAG_UNALLOC | FS_FLAG_ALIGN | FS_FLAG_META;
+			flags = FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT;
 			count = atoi(optarg);
 			break; 
 		case 'v':
@@ -178,10 +178,10 @@ main(int argc, char **argv)
     fs = fs_open(argv[optind++], fstype);
 
 	if (type == CALC_DLS)
-		fs->block_walk(fs, fs->start_block, fs->last_block,
+		fs->block_walk(fs, fs->first_block, fs->last_block,
 		  flags, count_dls, (char *) fs);
 	else if (type == CALC_DD)
-		fs->block_walk(fs, fs->start_block, fs->last_block,
+		fs->block_walk(fs, fs->first_block, fs->last_block,
 		  flags, count_dd, (char *) fs);
 
     fs->close(fs);
