@@ -2,7 +2,7 @@
 ** The Sleuth Kit
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -142,7 +142,7 @@ print_list_head(FS_INFO *fs, char *image)
     /*
      * Identify table type and table origin.
      */  
-    printf("class|host|image|start_time|unit\n");
+    printf("class|host|image|first_time|unit\n");
     printf("dls|%s|%s|%lu|%s\n", hostnamebuf, image, now, unit);
 
 	printf("addr|alloc\n");
@@ -152,7 +152,7 @@ print_list_head(FS_INFO *fs, char *image)
 static u_int8_t
 print_list(FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 {
-	printf("%lu|%s\n", (ULONG)addr, (flags & FS_FLAG_ALLOC)?"a":"f");
+	printf("%lu|%s\n", (ULONG)addr, (flags & FS_FLAG_DATA_ALLOC)?"a":"f");
 	return WALK_CONT;
 }
 
@@ -199,8 +199,7 @@ slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 
 	flen = fs_inode->size;
 	if ((fs->ftype & FSMASK) == FATFS_TYPE) {
-		fs->file_walk(fs, fs_inode, 0, 0, 
-		  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK),
+		fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK,
 		  slack_file_act, ptr);
 	}
 	else {
@@ -211,8 +210,7 @@ slack_inode_act(FS_INFO *fs, INUM_T inum, FS_INODE *fs_inode, int flags,
 
 			if (fs_data->flags & FS_DATA_NONRES) {
 				fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
-				  (FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK),
-				  slack_file_act, ptr);
+				  FS_FLAG_FILE_SLACK, slack_file_act, ptr);
 			}
 
 			fs_data = fs_data->next;
@@ -237,7 +235,7 @@ int     main(int argc, char **argv)
     DADDR_T bstart;
     DADDR_T blast;
     int     ch;
-    int     flags = FS_FLAG_UNALLOC | FS_FLAG_ALIGN | FS_FLAG_META; 
+    int     flags = FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALIGN | FS_FLAG_DATA_META | FS_FLAG_DATA_CONT; 
     char   *fstype = DEF_FSTYPE;
 	char	list = 0, slack = 0;
 
@@ -248,10 +246,10 @@ int     main(int argc, char **argv)
 	default:
 	    usage();
 	case 'b':
-	    flags &= ~FS_FLAG_ALIGN;
+	    flags &= ~FS_FLAG_DATA_ALIGN;
 	    break;
 	case 'e':
-	    flags |= FS_FLAG_ALLOC;
+	    flags |= FS_FLAG_DATA_ALLOC;
 	    break;
 	case 'f':
 	    fstype = optarg;
@@ -294,7 +292,7 @@ int     main(int argc, char **argv)
 		}
 	
 		fs->inode_walk(fs, fs->first_inum, fs->last_inum, 
-		  (FS_FLAG_ALLOC | FS_FLAG_USED | FS_FLAG_LINK | FS_FLAG_NOABORT), 
+		  (FS_FLAG_META_ALLOC | FS_FLAG_META_USED | FS_FLAG_META_LINK),
 		  slack_inode_act, (char *)0);
 	
 		exit(0);
@@ -309,9 +307,9 @@ int     main(int argc, char **argv)
     if (optind < argc) {
 	while ((start = argv[optind]) != 0) {
 	    last = split_at(start, '-');
-	    bstart = (*start ? atoblock(start) : fs->start_block);
-		if (bstart < fs->start_block)
-			bstart = fs->start_block;
+	    bstart = (*start ? atoblock(start) : fs->first_block);
+		if (bstart < fs->first_block)
+			bstart = fs->first_block;
 
 	    blast = (!last ? bstart : *last ? atoblock(last) : fs->last_block);
 		if (blast > fs->last_block)
@@ -330,10 +328,10 @@ int     main(int argc, char **argv)
      */
     else {
 		if (list)
-	    	fs->block_walk(fs, fs->start_block, fs->last_block, 
+	    	fs->block_walk(fs, fs->first_block, fs->last_block, 
 			   flags, print_list, (char *)fs);
 		else
-			fs->block_walk(fs, fs->start_block, fs->last_block,
+			fs->block_walk(fs, fs->first_block, fs->last_block,
 		       flags, print_block, (char *) fs);
     }
     fs->close(fs);
