@@ -1,11 +1,11 @@
 /*
 ** ext2fs_dent
-** The  Sleuth Kit 
+** The Sleuth Kit 
 **
 ** Human Interface Layer Support for an EXT2FS image
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -157,8 +157,8 @@ ext2fs_dent_copy(EXT2FS_INFO *ext2fs, char *ext2_dent, FS_DENT *fs_dent)
 **
 ** Read contents of directory block
 **
-** if entry is active call action with myflags set to FS_FLAG_ALLOC, if 
-** it is deleted then call action with FS_FLAG_UNALLOC.
+** if entry is active call action with myflags set to FS_FLAG_NAME_ALLOC, if 
+** it is deleted then call action with FS_FLAG_NAME_UNALLOC.
 ** len is the size of buf
 **
 ** return how much was read this time or 0 if action wanted to stop
@@ -242,16 +242,16 @@ ext2fs_dent_parse_block(EXT2FS_INFO *ext2fs, char *buf, int len,
 			myflags = 0;
 			/* Do we have a deleted entry? */
 			if ((idellen > 0) || (inode == 0)) {
-				myflags |= FS_FLAG_UNALLOC;
+				myflags |= FS_FLAG_NAME_UNALLOC;
 				idellen -= minreclen;
-				if (flags & FS_FLAG_UNALLOC)
+				if (flags & FS_FLAG_NAME_UNALLOC)
 					if (WALK_STOP == action(fs, fs_dent, myflags, ptr))
 						return 0;
 			}
 			/* We have a non-deleted entry */
 			else {
-				myflags |= FS_FLAG_ALLOC;
-				if (flags & FS_FLAG_ALLOC)
+				myflags |= FS_FLAG_NAME_ALLOC;
+				if (flags & FS_FLAG_NAME_ALLOC)
 					if (WALK_STOP == action(fs, fs_dent, myflags, ptr))
 						return 0;
 			}
@@ -273,8 +273,8 @@ ext2fs_dent_parse_block(EXT2FS_INFO *ext2fs, char *buf, int len,
 
 		
 			/* we will be recursing directories */
-			if ((myflags & FS_FLAG_ALLOC) &&
-			  (flags & FS_FLAG_RECURSE)  &&
+			if ((myflags & FS_FLAG_NAME_ALLOC) &&
+			  (flags & FS_FLAG_NAME_RECURSE)  &&
 			  (!ISDOT(fs_dent->name)) &&
 			  ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR)) {
 
@@ -328,12 +328,13 @@ ext2fs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 /* 
 ** The main function to do directory entry walking
 **
-** action is called for each entry with flags set to FS_FLAG_ALLOC for
+** action is called for each entry with flags set to FS_FLAG_NAME_ALLOC for
 ** active entries
 **
 ** this calls ext2fs_dent_walk_direct and _indirect
 **
-** Use the following flags: FS_FLAG_ALLOC, FS_FLAG_UNALLOC, FS_FLAG_RECURSE
+** Use the following flags: FS_FLAG_NAME_ALLOC, FS_FLAG_NAME_UNALLOC, 
+** FS_FLAG_NAME_RECURSE
 */
 void 
 ext2fs_dent_walk(FS_INFO *fs, INUM_T inode, int flags, 
@@ -357,8 +358,7 @@ ext2fs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
     curdirptr = dirbuf = mymalloc(size); 
     dirleft = size;
     
-    fs->file_walk(fs, fs_inode, 0, 0,
-      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK, 
+    fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK,
 	  ext2fs_dent_action, "");
 
     curdirptr = NULL;
