@@ -5,7 +5,7 @@
 ** Human interface Layer support for the FAT file system
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved 
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved 
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -38,21 +38,23 @@
  */ 
 
 /* Recursive path stuff */
-static unsigned int depth = 0;  /* how deep in the directory tree are we */
-#define MAX_DEPTH   64
-static char *didx[MAX_DEPTH];  /* pointer in dirs string to where '/' is for
-                        ** given depth */
 #define DIR_STRSZ   2048
-static char dirs[DIR_STRSZ];    /* The current directory name string */
+static char g_dirs[DIR_STRSZ];    /* The current directory name string */
+
+#define MAX_DEPTH   64
+static char *g_didx[MAX_DEPTH];  /* pointer in g_dirs string to where '/' is for
+                        ** given depth */
+static unsigned int g_depth = 0;  /* how deep in the directory tree are we */
 
 
 /* as FAT does not use inode numbers, we are making them up.  This causes
  * minor problems with the . and .. entries.  These global variables help
  * us out with that
  */
-static u_int32_t	p_inode = 0;	/* the . inode */
-static u_int32_t	pp_inode = 0;	/* the .. inode */
-static DADDR_T		pp_clust_tofind;		/* the first cluster that .. uses */
+static u_int32_t	g_curdir_inode = 0;	/* the . inode */
+static u_int32_t	g_pardir_inode = 0;	/* the .. inode */
+static DADDR_T		g_pardir_clust = 0;		/* the first cluster that .. uses */
+
 
 /* when set to 1, save data such as parent inodes and directory paths
  * in the global variables.  When set to 0 do not.  It is set to 0
@@ -61,7 +63,14 @@ static DADDR_T		pp_clust_tofind;		/* the first cluster that .. uses */
  *
  * Yes, I know this is a poor design 
  */
-static u_int8_t		save_state = 1;		
+static u_int8_t		g_save_state = 1;		
+
+
+
+/* Set to 1 when we are recursing down a deleted directory.  This will
+ * supress the errors that may occur from invalid data
+ */
+static u_int8_t		g_recdel = 0;
 
 
 /*
@@ -78,6 +87,10 @@ static u_int8_t		save_state = 1;
  * and then when we get the 8.3 entry the long name is copied in.  Therefore,
  * FATFS_INFO keeps state between calls to this function.
  *
+ *
+ * Ideally, we should print out the partial lfn when we find one with a
+ * new checksum, but that does not work because this is supposed to return
+ * only one entry and that case may cause two entries to be returned.
  */
 static void 
 fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent, 
@@ -99,6 +112,7 @@ fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent,
 		/* Is this a new sequence?  The checksum is the same for all entries
 		 * in the same sequence 
 		 */
+
 		if (dirl->chksum != fatfs->lfn_chk) { 
 			fatfs->lfn_seq = dirl->seq;
 			fatfs->lfn_chk = dirl->chksum;
@@ -125,6 +139,29 @@ fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent,
 			}
 		}
 	}
+	/* Special case for volume label, where name does not have an
+	 * extension and we add a note at the end that it is a label */
+	else if ((dir->attrib & FATFS_ATTR_VOLUME) == FATFS_ATTR_VOLUME) {
+		fs_dent->namlen = 0;
+		for (i = 0 ; i < 8; i++) {
+			if ((dir->name[i] != 0) && (dir->name[i] != 0xff)) {
+				fs_dent->name[fs_dent->namlen++] = dir->name[i];
+			}
+		}
+		for (i = 0; i < 3; i++) {
+			if ((dir->ext[i] != 0) && (dir->ext[i] != 0xff)) {
+				fs_dent->name[fs_dent->namlen++] = dir->ext[i];
+			}
+		}
+
+		fs_dent->name[fs_dent->namlen] = '\0';	
+		/* Append a string to show it is a label */
+		if (fs_dent->namlen + 22 < FATFS_MAXNAMLEN) {
+			char *volstr = " (Volume Label Entry)";
+			strncat (fs_dent->name, volstr, FATFS_MAXNAMLEN - fs_dent->namlen);
+			fs_dent->namlen += strlen(volstr);
+		}
+	}
 
 	/* we have a short (8.3) entry 
 	 * we may have the long name version already stored in the FATFS_INFO
@@ -134,6 +171,7 @@ fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent,
 		fs_dent->namlen = 0;
 
 		/* if we have the lfn, copy it in.  Remember it is backwards */
+		/* @@@ We could check the checksum, but how would we report it */
 		if (fatfs->lfn_len != 0) {
 			for (i = 0; i < fatfs->lfn_len; i++) 
 				fs_dent->name[fs_dent->namlen++] = 
@@ -192,8 +230,8 @@ fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent,
 
 
 	/* append the path data */
-	fs_dent->path = dirs;
-	fs_dent->pathdepth = depth;
+	fs_dent->path = g_dirs;
+	fs_dent->pathdepth = g_depth;
 
 
 	/* file type: FAT only knows DIR and FILE */
@@ -225,14 +263,15 @@ fatfs_dent_copy(FATFS_INFO *fatfs, char *fatfs_dent, FS_DENT *fs_dent,
  * this is the call back for the dent walk when we need to find the 
  * parent directory
  *
- * save_state should be 0 when this is running
+ * g_save_state should be 0 when this is running
  */
 static u_int8_t
 find_parent_act(FS_INFO *fs, FS_DENT *fsd, int flags, char *ptr)
 {
-	/* we found the entry that points to the requested clustor */
-	if (fsd->fsi->direct_addr[0] == pp_clust_tofind) {
-		pp_inode = fsd->inode;
+	/* we found the directory entry that has allocated the cluster 
+	 * we are looking for */
+	if (fsd->fsi->direct_addr[0] == g_pardir_clust) {
+		g_pardir_inode = fsd->inode;
 		return WALK_STOP;
 	}
 	return WALK_CONT;
@@ -240,9 +279,10 @@ find_parent_act(FS_INFO *fs, FS_DENT *fsd, int flags, char *ptr)
 
 /*
  * this function will find the parent inode of a given directory
- * as specified in fs_dent
+ * as specified in fs_dent. It works by walking the directory tree
+ * starting at the root.  
  *
- * Set save_state to 0, so that no global variables are overwritten.  I do
+ * Set g_save_state to 0, so that no global variables are overwritten.  I do
  * not like the design of this, but this is much more efficient than
  * an inode walk
  *
@@ -253,29 +293,30 @@ find_parent (FATFS_INFO *fatfs, FS_DENT *fs_dent)
 {
 	FS_INFO *fs = (FS_INFO *)&fatfs->fs_info;	
 
-	if (save_state == 0)
+	if (g_save_state == 0)
 		error ("save_state is already 0, recursion?");
 
-	pp_inode = 0;
 
 	/* set the global value that the action function will use */
-	pp_clust_tofind = fs_dent->fsi->direct_addr[0];
+	g_pardir_inode = 0;
+	g_pardir_clust = fs_dent->fsi->direct_addr[0];
+
 
 	/* is the root directory being called for? 
 	 * we won't find it by searching as it does not exist
 	 */
 	if (fs->ftype == MS32_FAT) {
-		OFF_T clust = 2 + (fatfs->rootsect - fatfs->firstclustsect) / 
-		  fatfs->csize;	
-		if ((clust == pp_clust_tofind) || (pp_clust_tofind == 0)) {
-			pp_inode = fs->root_inum;
-			return pp_inode;
+		OFF_T clust = FATFS_SECT_2_CLUST (fatfs, fatfs->rootsect);
+
+		if ((clust == g_pardir_clust) || (g_pardir_clust == 0)) {
+			g_pardir_inode = fs->root_inum;
+			return g_pardir_inode;
 		}
 	}	
 	else {
-		if ((pp_clust_tofind == 1) || (pp_clust_tofind == 0)) {
-			pp_inode = fs->root_inum;
-			return pp_inode;
+		if ((g_pardir_clust == 1) || (g_pardir_clust == 0)) {
+			g_pardir_inode = fs->root_inum;
+			return g_pardir_inode;
 		}
 	}
 
@@ -288,13 +329,14 @@ find_parent (FATFS_INFO *fatfs, FS_DENT *fs_dent)
 	/* walk the inodes - looking for an inode that has allocated the
 	 * same first sector 
 	 */
-	save_state = 0;
-	fatfs_dent_walk(fs, fs->first_inum, FS_FLAG_ALLOC | FS_FLAG_RECURSE,
-		find_parent_act, "find_parent");
-	save_state = 1;
+	g_save_state = 0;
+	fatfs_dent_walk(fs, fs->root_inum, 
+	  FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_RECURSE,
+	  find_parent_act, "find_parent");
+	g_save_state = 1;
 
 	/* if we didn't find anything then 0 will be returned */
-	return pp_inode;
+	return g_pardir_inode;
 }
 
 /* 
@@ -304,7 +346,7 @@ find_parent (FATFS_INFO *fatfs, FS_DENT *fs_dent)
 **
 ** The length read is returned or 0 if the action wanted to stop.  
 **
-** flags: FS_FLAG_ALLOC, FS_FLAG_RECURSE
+** flags: FS_FLAG_NAME_ALLOC, FS_FLAG_NAME_UNALLOC, FS_FLAG_NAME_RECURSE
 */
 static int 
 fatfs_dent_parse_block(FATFS_INFO *fatfs, char *buf, int len, DADDR_T addr, 
@@ -315,6 +357,7 @@ fatfs_dent_parse_block(FATFS_INFO *fatfs, char *buf, int len, DADDR_T addr,
 	fatfs_dentry	*dep;
 	FS_DENT 	*fs_dent;
 	FS_INFO 	*fs = (FS_INFO *)&fatfs->fs_info;
+	int			sectalloc;
 
 	fs_dent = fs_dent_alloc(FATFS_MAXNAMLEN);
 
@@ -323,135 +366,151 @@ fatfs_dent_parse_block(FATFS_INFO *fatfs, char *buf, int len, DADDR_T addr,
 	/* Get the base inode for this sector */
 	ibase = FATFS_SECT_2_INODE(fatfs, addr);
 	if (ibase > fs->last_inum)
-		error("parse error: inode is too large");
+		error("parse error: inode address is too large");
 
-	/* cycle through the entries */
+	sectalloc = is_sectalloc (fatfs, addr);
+
+	/* cycle through the directory entries */
 	for (idx = 0; idx < fatfs->dentry_cnt_se; idx++, dep++)  {
 
-			int myflags = 0;
+		int myflags = 0;
 
-			/* is it a valid dentry? */
-			if (0 == fatfs_isdentry(fatfs, dep))
-				continue;
+		/* is it a valid dentry? */
+		if (0 == fatfs_isdentry(fatfs, dep))
+			continue;
 
-			/* if a volume label, then skip it */
-			if (((dep->attrib & FATFS_ATTR_VOLUME) == FATFS_ATTR_VOLUME) &&
-			   ((dep->attrib & FATFS_ATTR_LFN) != FATFS_ATTR_LFN))
-				continue;
+		inode = ibase + idx;
 
-			inode = ibase + idx;
+		/* Copy the entry
+		 * if this is LFN, then all that will happen is that it will
+		 * be copied into FATFS_INFO 
+		 */
+		fatfs_dent_copy(fatfs, (char *)dep, fs_dent, inode);
 
 
-			/* Copy the entry
-			 * if this is LFN, then all that will happen is that it will
-			 * be copied into FATFS_INFO 
-			 */
-			fatfs_dent_copy(fatfs, (char *)dep, fs_dent, inode);
+		/* If it is a long file name, then keep on going until the
+		 * final small name is reached */
+		if ((dep->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN) 
+			continue;
 
+		/* Handle the . and .. entries specially
+		 * The current inode 'address' they have is for the current
+		 * slot in the cluster, but it needs to refer to the original
+		 * slot 
+		 */
+		if (dep->name[0] == '.') {
 
-			/* If it is a long file name, then keep on going until the
-			 * final small name is reached */
-			if ((dep->attrib & FATFS_ATTR_LFN) == FATFS_ATTR_LFN) 
-				continue;
+			/* g_curdir_inode is always set and we can copy it in */
+			if (dep->name[1] == ' ') 
+				inode = fs_dent->inode = g_curdir_inode;
 
-			/* Handle the . and .. entries specially
-			 * The current inode 'address' they have is for the current
-			 * slot in the cluster, but it needs to refer to the original
-			 * slot 
-			 */
-			if (dep->name[0] == '.') {
+			/* g_pardir_inode is not always set, so we may have to search */
+			else if (dep->name[1] == '.') {
 
-				/* p_inode is always set and we can copy it in */
-				if (dep->name[1] == ' ') 
-					inode = fs_dent->inode = p_inode;
+				/* g_save_state is set to 0 when we are already looking
+				 * for a parent */
+				if ((!g_pardir_inode) && (g_save_state))
+					g_pardir_inode = find_parent(fatfs, fs_dent);
 
-				/* pp_inode is not always set, so we may have to search */
-				else if (dep->name[1] == '.') {
-					/* save_state is set to 0 when we are already looking
-					 * for a parent */
-					if ((!pp_inode) && (save_state))
-						pp_inode = find_parent(fatfs, fs_dent);
+				inode = fs_dent->inode = g_pardir_inode;
 
-					inode = fs_dent->inode = pp_inode;
-
-					/* If the parent is the root, the data is invalid
-					 * so make it up
-					 */
-					if (inode == fs->root_inum) 
-						fatfs_make_root(fatfs, fs_dent->fsi);
-				}
+				/* If the .. entry is for the root directory, then make
+				 * up the data
+				 */
+				if (inode == fs->root_inum) 
+					fatfs_make_root(fatfs, fs_dent->fsi);
 			}
+		}
 
-			
-			/* Do we have a deleted entry? */
+
+		/* The allocation status of an entry is based on the allocation
+		 * status of the sector it is in and the flag.  Deleted directories
+		 * do not always clear the flags of each entry
+		 */
+		if (sectalloc == 1) {
 			myflags = (dep->name[0] == FATFS_SLOT_DELETED)?
-			  FS_FLAG_UNALLOC : FS_FLAG_ALLOC;
+			  FS_FLAG_NAME_UNALLOC : FS_FLAG_NAME_ALLOC;
+		}
+		else {
+			myflags = FS_FLAG_NAME_UNALLOC;
+		}
+
+		if ((flags & myflags) == myflags) {
+			if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
+				fs_dent_free(fs_dent);
+				return 0;
+			}
+		}
+
+
+		/* if we have a directory and need to recurse then do it */
+		if (((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR) && 
+		  (flags & FS_FLAG_NAME_RECURSE)  &&
+		  (!ISDOT(fs_dent->name)) ) {
+
+			u_int32_t back_p = 0;
+			u_int8_t back_recdel = 0;
+
+
+			/* we are going to append only the short name */
+			if ((g_save_state) && (g_depth < MAX_DEPTH)) {
+				char tmpname[13];
+					
+				if ((strlen(fs_dent->name) > 12) || 
+				  (strrchr(fs_dent->name, '('))) {
+					int i;
+					char *ptr;
+
+					ptr = strrchr (fs_dent->name, '(');
+					ptr++;
+
+					for (i = 0; i < 12 && ptr[i] != ')'; i++) 
+						tmpname[i] = ptr[i];
+				
+					tmpname[i] = '\0';
 
-			if ((flags & myflags) == myflags) {
-				if (WALK_STOP == action(fs, fs_dent, myflags, ptr)) {
-					fs_dent_free(fs_dent);
-					return 0;
 				}
+				else {
+					strncpy (tmpname, fs_dent->name, 13);
+				}
+
+				g_didx[g_depth] = &g_dirs[strlen(g_dirs)];
+				strncpy(g_didx[g_depth], tmpname, DIR_STRSZ - strlen(g_dirs));
+				strncat(g_dirs, "/", DIR_STRSZ);
 			}
 
+			/* save the .. inode value */
+			if (g_save_state) {
+				back_p = g_pardir_inode;
+				g_pardir_inode = g_curdir_inode;
+				g_depth++;
+			}
 
-			/* if we have an allocated entry and we need to recurse,
-			 * then do it 
-			 */
-            if ((myflags & FS_FLAG_ALLOC) &&
-              (flags & FS_FLAG_RECURSE)  &&
-              (!ISDOT(fs_dent->name)) &&
-              ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR)) {
+			/* This will prevent errors from being generated from the invalid
+			 * deleted files.  save the current setting and set it to del */
+			if (myflags & FS_FLAG_NAME_ALLOC)  {
+				back_recdel = g_recdel;
+				g_recdel = 1;
+			}
 
-				u_int32_t back_p = 0;
+			fatfs_dent_walk(&(fatfs->fs_info), fs_dent->inode, 
+			  flags, action, ptr);
 
-				/* we are going to append only the short name */
-                if ((save_state) && (depth < MAX_DEPTH)) {
-					char tmpname[13];
-					
-					if ((strlen(fs_dent->name) > 12) || 
-					  (strrchr(fs_dent->name, '('))) {
-						int i;
-						char *ptr;
+			if (g_save_state) {
+				g_depth--;
+				g_curdir_inode = g_pardir_inode;
+				g_pardir_inode = back_p; 
+			}
 
-						ptr = strrchr (fs_dent->name, '(');
-						ptr++;
+			if ((g_save_state) && (g_depth < MAX_DEPTH))
+				 *g_didx[g_depth] = '\0';
 
-						for (i = 0; i < 12 && ptr[i] != ')'; i++) 
-							tmpname[i] = ptr[i];
-					
-						tmpname[i] = '\0';
+			/* Restore the recursion setting */
+			if (myflags & FS_FLAG_NAME_ALLOC)  {
+				g_recdel = back_recdel;
+			}
 
-					}
-					else {
-						strncpy (tmpname, fs_dent->name, 13);
-					}
-
-                    didx[depth] = &dirs[strlen(dirs)];
-					strncpy(didx[depth], tmpname, DIR_STRSZ - strlen(dirs));
-					strncat(dirs, "/", DIR_STRSZ);
-				}
-
-				/* save the .. inode value */
-				if (save_state) {
-					back_p = pp_inode;
-					pp_inode = p_inode;
-					depth++;
-				}
-
-				fatfs_dent_walk(&(fatfs->fs_info), fs_dent->inode, 
-				  flags, action, ptr);
-
-				if (save_state) {
-					depth--;
-					p_inode = pp_inode;
-					pp_inode = back_p; 
-				}
-
-				if ((save_state) && (depth < MAX_DEPTH))
-					 *didx[depth] = '\0';
-
-        	} /* end of recurse */
+		} /* end of recurse */
 
 	} /* end for dentries */
 
@@ -469,35 +528,52 @@ fatfs_dent_parse_block(FATFS_INFO *fatfs, char *buf, int len, DADDR_T addr,
  *************************************************************************/
 
 /* values used to copy the directory contents into a buffer */
-static char *curdirptr = NULL;
-static int dirleft = 0;
-static DADDR_T *curaddrbuf = NULL;
-static int addralloc = 0;
-static int addridx = 0;
-#define DENT_ADDR_INC 512
 
+/* ptr to the current location in a local buffer */
+static char *g_curdirptr = NULL;		
+
+/* number of bytes left in g_curdirptr */
+static int g_dirleft = 0;				
+
+
+/* ptr to a local buffer for the stack of sector addresses */
+static DADDR_T *g_curaddrbuf = NULL;
+
+/* num of entries allocated to g_curaddrbuf */
+static int g_addrsize = 0;
+
+/* The current index in the g_curaddrbuf stack */
+static int g_addridx = 0;
+
+
+/* 
+ * file_walk callback action to load directory contents 
+ */
 static u_int8_t 
 fatfs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
   int flags, char *ptr)
 {       
-	/* how much are we copying */
-    int len = (dirleft < size) ? dirleft : size;
-    memcpy (curdirptr, buf, len);
-    curdirptr = (char *) ((int)curdirptr + len);
-    dirleft -= len;
+	/* how much of the buffer are we copying */
+    int len = (g_dirleft < size) ? g_dirleft : size;
+
+	/* Copy the sector into a buffer and increment the pointers */
+    memcpy (g_curdirptr, buf, len);
+    g_curdirptr = (char *) ((int)g_curdirptr + len);
+    g_dirleft -= len;
 
 	/* fill in the stack of addresses of sectors 
 	 *
-	 * if we are at the end of our buffer, realloc more */
-	if (addridx == addralloc) {
-		addralloc += DENT_ADDR_INC;
-		curaddrbuf = (DADDR_T *)myrealloc ((char *)curaddrbuf, 
-		  addralloc * sizeof(DADDR_T));
+	 * if we are at the last entry, then realloc more */
+	if (g_addridx == g_addrsize) {
+		g_addrsize += 512;
+		g_curaddrbuf = (DADDR_T *)myrealloc ((char *)g_curaddrbuf, 
+		  g_addrsize * sizeof(DADDR_T));
 	}
 
-	curaddrbuf[addridx++] = addr;
+	/* Add this sector to the stack */
+	g_curaddrbuf[g_addridx++] = addr;
 
-    if (dirleft)
+    if (g_dirleft)
         return WALK_CONT;
     else   
         return WALK_STOP;
@@ -507,10 +583,11 @@ fatfs_dent_action(FS_INFO *fs, DADDR_T addr, char *buf, int size,
 /* 
 ** The main function to do directory entry walking
 **
-** action is called for each entry with flags set to FS_FLAG_ALLOC for
+** action is called for each entry with flags set to FS_FLAG_NAME_ALLOC for
 ** active entries
 **
-** Use the following flags: FS_FLAG_ALLOC, FS_FLAG_UNALLOC, FS_FLAG_RECURSE
+** Use the following flags: FS_FLAG_NAME_ALLOC, FS_FLAG_NAME_UNALLOC, 
+** FS_FLAG_NAME_RECURSE
 */
 void 
 fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags, 
@@ -529,13 +606,13 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 
 	fs_inode = fs->inode_lookup(fs, inode);
 	if (!fs_inode)
-		error ("not an inode");
+		error ("%lu is not a valid inode", (ULONG)inode);
 		
 	size = fs_inode->size;
 
 	/* If we are saving state, save the current inode value ('.') */
-	if (save_state)
-		p_inode = inode;
+	if (g_save_state)
+		g_curdir_inode = inode;
 
 	/* Allocate a buffer for the lfn support */
 	if (fatfs->lfn == NULL) {
@@ -545,40 +622,80 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 		fatfs->lfn_seq = 0;
 	}
 
-    /* make a copy of the directory contents that we can process */
-    if (curdirptr != NULL)
-        error ("fatfs_dent_walk: Curdirptr is set! recursive?");
 
-    if (curaddrbuf != NULL)
-        error ("fatfs_dent_walk: Curaddrbuf is set! recursive?");
+	/* Make a copy of the directory contents.  This will use
+	 * the file_walk functionality 
+	 */
 
-	/* make a buffer for the directory */
-    curdirptr = dirbuf = mymalloc(size);
-    dirleft = size;
 
-	/* make a stack of addresses */
-	addralloc = DENT_ADDR_INC;
-    curaddrbuf = addrbuf = (DADDR_T *)mymalloc(addralloc * sizeof(DADDR_T));
-	addridx = 0;
+	/* Allocate a buffer for the directory contents */
+    if (g_curdirptr != NULL)
+        error ("fatfs_dent_walk: g_curdirptr is set! recursive?");
+
+	dirbuf = mymalloc(size);
+
+	/* Set the global variables that are used during the copy */
+    g_curdirptr = dirbuf;
+    g_dirleft = size;
+
+
+	/* We are going to save the address of each sector in the directory
+	 * in a stack - they are needed to determine the inode address */
+	if (g_curaddrbuf != NULL)
+		error ("fatfs_dent_walk: g_curaddrbuf is set! recursive?");
+
+	g_addrsize = 512;
+	addrbuf = (DADDR_T *)mymalloc(g_addrsize * sizeof(DADDR_T));
+
+	/* Set the global variables that are used during the copy */
+	g_addridx = 0;			
+	g_curaddrbuf = addrbuf;	
   
+
 	/* save the directory contents into dirbuf */
-    fs->file_walk(fs, fs_inode, 0, 0,
-      FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_SLACK, 
-	  fatfs_dent_action, "");
+	if (g_recdel == 0) {
+    	fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_SLACK,
+		  fatfs_dent_action, "");
 
-	addrmax = addridx;
-	/* the pointer could have moved due to a realloc */
-	addrbuf = curaddrbuf;
+		if (g_dirleft > 0) {
+			error ("fatfs_dent_walk: Entire directory was not loaded (%d left)",
+			  g_dirleft);
+		}
+	} 
+	else { 
+    	fs->file_walk(fs, fs_inode, 0, 0, 
+		  FS_FLAG_FILE_SLACK | FS_FLAG_FILE_RECOVER,
+		  fatfs_dent_action, "");
 
-    curdirptr = NULL;
- 	curaddrbuf = NULL;
-    dirptr = dirbuf;
+		/* We did not copy the entire directory 
+		 * If we are recursing on a deleted directory, then exit now.
+		 * during recovery of a deleted file, the FAT code will only
+		 * return content if it can recover the entire file
+		 */
+		if (g_dirleft > 0) {
+			return;
+		}
+	}
+
+
+	/* How many sector addresses do we have? */
+	addrmax = g_addridx;
+
+	/* Reset the local pointer because we could have done a realloc */
+	addrbuf = g_curaddrbuf;
+
+	/* Reset the global pointers */
+    g_curdirptr = NULL;
+ 	g_curaddrbuf = NULL;
 
 	/* cycle through the directory and parse the contents */
+    dirptr = dirbuf;
 	for (i = 0; size > 0; i++) {
         int len = (fatfs->ssize < size) ? fatfs->ssize : size;
 		int retval;
 
+		/* This should never happen because we wouldn't have been able
+		 * to load the directory, but it never hurts ... */
 		if (addrbuf[i] > fs->last_block)
 			error ("fatfs_dent_walk: block too large");
 
@@ -591,8 +708,13 @@ fatfs_dent_walk(FS_INFO *fs, INUM_T inode, int flags,
 		else
 			break;
 
-		if (len != retval)
-			error ("hmm, funky return");
+		if (len != retval) {
+			if (g_recdel == 1)
+				return;
+
+			error ("fatfs_dent_parse: returned non-zero size: %lu %lu",
+			  (ULONG)len, (ULONG)retval);
+		}
 
         dirptr = (char *)((int)dirptr + len);
     }
