@@ -319,15 +319,15 @@ ffs_copy_inode(FFS_INFO *ffs, FS_INODE *fs_inode)
 
 	/* get the alloc flag */
 	fs_inode->flags = (isset(inosused, ffs->inum - ibase) ?
-	  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+	  FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
 	/* link flag */
 	fs_inode->flags |= (fs_inode->nlink ?
-	  FS_FLAG_LINK : FS_FLAG_UNLINK);
+	  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
   
 	/* used/unused */
 	fs_inode->flags |= (fs_inode->ctime ? 
-	  FS_FLAG_USED : FS_FLAG_UNUSED);
+	  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 
 } /* end of ffs_copy_inode */
 
@@ -362,7 +362,9 @@ ffs_inode_lookup(FS_INFO *fs, INUM_T inum)
 
 /* ffs_inode_walk - inode iterator 
  *
- * flags used: USED, UNUSED, LINK, UNLINK, ALLOC, UNALLOC
+ * flags used: FS_FLAG_META_USED, FS_FLAG_META_UNUSED, 
+ *  FS_FLAG_META_LINK, FS_FLAG_META_UNLINK, 
+ *  FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC
  */
 void
 ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
@@ -408,7 +410,7 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 		 * Apply the allocated/unallocated restriction.
 		 */
 		myflags = (isset(inosused, inum - ibase) ?
-			   FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+			   FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 		if ((flags & myflags) != myflags)
 			continue;
 
@@ -420,14 +422,16 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 		/*
 		 * Apply the linked/unlinked restriction.
 		 */
-		myflags |= (gets16(fs, in->di_nlink) ? FS_FLAG_LINK : FS_FLAG_UNLINK);
+		myflags |= (gets16(fs, in->di_nlink) ? 
+		  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 		if ((flags & myflags) != myflags)
 			continue;
 
 		/*
 		 * Apply the used/unused restriction.
 		 */
-		myflags |= (gets32(fs, in->di_ctime) ? FS_FLAG_USED : FS_FLAG_UNUSED);
+		myflags |= (gets32(fs, in->di_ctime) ? 
+		  FS_FLAG_META_USED : FS_FLAG_META_UNUSED);
 		if ((flags & myflags) != myflags)
 			continue;
 
@@ -458,7 +462,8 @@ ffs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 /* ffs_block_walk - block iterator 
  *
- * flags used: ALIGN, META, ALLOC, UNALLOC
+ * flags: FS_FLAG_DATA_ALLOC, FS_FLAG_DATA_UNALLOC, FS_FLAG_DATA_CONT,
+ *  FS_FLAG_DATA_META, FS_FLAG_DATA_ALIGN
  */
 
 void
@@ -467,7 +472,7 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 {
     char   *myname = "ffs_block_walk";
     FFS_INFO *ffs = (FFS_INFO *) fs;
-    FS_BUF *fs_buf = fs_buf_alloc(fs->block_size * fs->block_frags);
+    FS_BUF *fs_buf = fs_buf_alloc(fs->block_size * ffs->block_frags);
     CGNUM_T cg_num;
     ffs_cg *cg = 0;
     DADDR_T dbase = 0;
@@ -484,19 +489,19 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     /*
      * Sanity checks.
      */
-    if (start < fs->start_block || start > fs->last_block)
+    if (start < fs->first_block || start > fs->last_block)
 		error("%s: invalid start block number: %lu", myname, (ULONG) start);
 
-    if (last < fs->start_block || last > fs->last_block || last < start)
+    if (last < fs->first_block || last > fs->last_block || last < start)
 		error("%s: invalid last block number: %lu", myname, (ULONG) last);
 
-    if ((flags & FS_FLAG_ALIGN) && (start % fs->block_frags) != 0)
+    if ((flags & FS_FLAG_DATA_ALIGN) && (start % ffs->block_frags) != 0)
 		error("%s: specify -b or specify block-aligned start block", myname);
 
     /*
      * Other initialization.
      */
-    if (flags & FS_FLAG_ALIGN) {
+    if (flags & FS_FLAG_DATA_ALIGN) {
 		null_block = mymalloc(fs->block_size);
 		memset(null_block, 0, fs->block_size);
     }
@@ -510,7 +515,7 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
      * composed of a number of fragment blocks. For example, the 4.4BSD
      * filesystem has logical blocks of 8 fragments.
      */
-    for (addr = start; addr <= last; addr += fs->block_frags) {
+    for (addr = start; addr <= last; addr += ffs->block_frags) {
 
 	/*
 	 * Be sure to use the right cylinder group information.
@@ -532,8 +537,8 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 	/*
 	 * Prepare for file systems that have a partial last logical block.
 	 */
-	frags = (last + 1 - addr > fs->block_frags ?
-		 fs->block_frags : last + 1 - addr);
+	frags = (last + 1 - addr > ffs->block_frags ?
+		 ffs->block_frags : last + 1 - addr);
 
 	/*
 	 * See if this logical block contains any fragments of interest. If
@@ -541,7 +546,7 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 	 */
 	for (want = 0, faddr = addr; want == 0 && faddr < addr + frags; faddr++)
 	    want = (flags & (isset(freeblocks, faddr - dbase) ?
-			     FS_FLAG_UNALLOC : FS_FLAG_ALLOC));
+			     FS_FLAG_DATA_UNALLOC : FS_FLAG_DATA_ALLOC));
 	if (want == 0)
 	    continue;
 
@@ -555,24 +560,27 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 	 */
 	for (faddr = addr; faddr < addr + frags; faddr++) {
 	    myflags = (isset(freeblocks, faddr - dbase) ?
-		       FS_FLAG_UNALLOC : FS_FLAG_ALLOC);
+		       FS_FLAG_DATA_UNALLOC : FS_FLAG_DATA_ALLOC);
         if (faddr >= sblock && faddr < dmin)
-        	myflags |= FS_FLAG_META;
+        	myflags |= FS_FLAG_DATA_META;
+		else
+			myflags |= FS_FLAG_DATA_CONT;
 
-        if ((myflags & FS_FLAG_META) && (myflags & FS_FLAG_UNALLOC))
+        if ((myflags & FS_FLAG_DATA_META) && (myflags & FS_FLAG_DATA_UNALLOC))
 			remark("impossible: unallocated meta block %lu!!",
 				   (unsigned long) faddr);
+
 	    if ((flags & myflags) != myflags) {
 			/* we dont' want this fragment, but there is another we want,
 			 * so we only print it if ALIGN is set */
-			if (flags & FS_FLAG_ALIGN)
+			if (flags & FS_FLAG_DATA_ALIGN)
 		    	if (WALK_STOP == action(fs, faddr, null_block, myflags, ptr)) {
 					free(null_block);
     				fs_buf_free(fs_buf);
 					return;
 				}
 	    } else {
-			if (fs_buf->addr < 0 || faddr >= fs_buf->addr + fs->block_frags) {
+			if (fs_buf->addr < 0 || faddr >= fs_buf->addr + ffs->block_frags) {
 				fs_read_block(fs, fs_buf, fs->block_size * frags, addr,
 				  "data block");
 			}
@@ -589,7 +597,7 @@ ffs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     /*
      * Cleanup.
      */
-    if (flags & FS_FLAG_ALIGN)
+    if (flags & FS_FLAG_DATA_ALIGN)
 		free(null_block);
     fs_buf_free(fs_buf);
 }
@@ -613,7 +621,7 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
 	read_count = (length < buf->size ? length : buf->size);
 
     if (addr > fs->last_block) {
-        if (flags & FS_FLAG_NOABORT)  {
+        if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
 				  "Invalid direct block address (too large): %lu",
@@ -636,30 +644,26 @@ ffs_file_walk_direct(FS_INFO *fs, FS_BUF *buf,
 		read_count = (fs->last_block - addr + 1) * fs->block_size;
 	}
 
-	/* all direct pointers are content */
-    myflags = FS_FLAG_CONT;
-
     if (addr == 0) {
-        if (flags & FS_FLAG_UNALLOC) {
-			myflags |= FS_FLAG_UNALLOC;
+		if (0 == (flags & FS_FLAG_FILE_NOSPARSE)) {
+			myflags = FS_FLAG_DATA_CONT;
 
-            if ((flags & FS_FLAG_AONLY) == 0)
+            if ((flags & FS_FLAG_FILE_AONLY) == 0)
                 memset(buf->data, 0, read_count);
+
             if (WALK_STOP == action(fs, addr, buf->data, read_count, 
-			  myflags, ptr)) 
+			   myflags, ptr)) 
 				return 0;
         }
     } else {
-        if (flags & FS_FLAG_ALLOC) {
-			myflags |= FS_FLAG_ALLOC;
+		myflags = FS_FLAG_DATA_CONT;
 
-            if ((flags & FS_FLAG_AONLY) == 0)
-                fs_read_block(fs, buf, roundup(read_count, FFS_DEV_BSIZE), addr,
-                      "data block");
-            if (WALK_STOP == action(fs, addr, buf->data, read_count, 
-			  myflags, ptr))
-				return 0;
-        }
+		if ((flags & FS_FLAG_FILE_AONLY) == 0)
+			fs_read_block(fs, buf, roundup(read_count, FFS_DEV_BSIZE), addr,
+			  "data block");
+
+		if (WALK_STOP == action(fs, addr, buf->data, read_count, myflags, ptr))
+			return 0;
     }
     return (read_count);
 }
@@ -683,7 +687,7 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 ;   
     
     if (addr > fs->last_block) {
-        if (flags & FS_FLAG_NOABORT)  {
+        if (flags & FS_FLAG_FILE_NOABORT)  {
 			if (verbose) {
 				fprintf (logfp, 
         		  "Invalid indirect block address (too large): %lu",
@@ -707,8 +711,8 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
               "disk address block");
 
 	/* we only call the action  if the META flag is set */
-    if (flags & FS_FLAG_META) {
-        int myflags = FS_FLAG_META | FS_FLAG_ALLOC;   
+    if (flags & FS_FLAG_FILE_META) {
+        int myflags = FS_FLAG_DATA_META;
         action(fs, addr, buf[level]->data, buf[level]->size, myflags, ptr);
     }
 
@@ -721,7 +725,7 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 		int prevcnt = todo_count;
 
 		if (getu32(fs, (u_int8_t *)&iaddr[n]) > fs->last_block) {
-			if (flags & FS_FLAG_NOABORT) {
+			if (flags & FS_FLAG_FILE_NOABORT) {
 				if (verbose) {
 					fprintf (logfp, 
 					  "Invalid address in indirect list (too large): %lu",
@@ -752,8 +756,13 @@ ffs_file_walk_indir(FS_INFO *fs, FS_BUF *buf[], size_t length,
 
 /*  
  *  
- * flag values: FS_FLAG_{UN}ALLOC, FS_FLAG_META, FS_FLAG_CONT, FS_FLAG_AONLY
- * FS_FLAG_SLACK
+ * flag values: FS_FLAG_FILE_SPARSE, FS_FLAG_FILE_AONLY, FS_FLAG_FILE_SLACK
+ *  FS_FLAG_FILE_NOABORT, FS_FLAG_FILE_META
+ *
+ * nothing special is done for FS_FLAG_FILE_RECOVER
+ *
+ * Action uses: FS_FLAG_DATA_CONT, FS_FLAG_DATA_META
+ * @@@ DATA_ALLOC and _UNALLOC are not implemented
  *  
  * The type and id fields are ignored with FFS
  */ 
@@ -777,7 +786,7 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
     
     length = inode->size;
 	/* If we want the slack of the last fragment, then roundup */
-    if (flags & FS_FLAG_SLACK)
+    if (flags & FS_FLAG_FILE_SLACK)
 		length = roundup(length, fs->block_size);
 
     /*
@@ -799,7 +808,7 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
 
 		/* allocate buffers */
 		for (level = 1; level <= inode->indir_count; level++)
-			buf[level] = fs_buf_alloc(fs->addr_bsize);
+			buf[level] = fs_buf_alloc(fs->file_bsize);
 
 		for (level = 1; length > 0 && level <= inode->indir_count; level++) {
 			retval = ffs_file_walk_indir(fs, buf, length,
@@ -824,6 +833,11 @@ ffs_file_walk(FS_INFO *fs, FS_INODE *inode, u_int32_t type, u_int16_t id,
 }
 
 
+static void
+ffs_fscheck(FS_INFO *fs, FILE *hFile)
+{
+	error ("fscheck not implemented for ffs yet");
+}
 
 static void
 ffs_fsstat(FS_INFO *fs, FILE *hFile)
@@ -855,7 +869,7 @@ ffs_fsstat(FS_INFO *fs, FILE *hFile)
     fprintf(hFile, "--------------------------------------------\n");
 
 	fprintf(hFile, "Fragment Range: %lu - %lu\n", 
-	  (ULONG)fs->start_block, (ULONG)fs->last_block);
+	  (ULONG)fs->first_block, (ULONG)fs->last_block);
 
 	fprintf(hFile, "Block Size: %lu\n", (ULONG)fs->file_bsize);
 	fprintf(hFile, "Fragment Size: %lu\n", (ULONG)fs->block_size);
@@ -951,7 +965,7 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
 {   
 	FILE *hFile = (FILE *)ptr;
 
-    if (flags & FS_FLAG_CONT) {
+    if (flags & FS_FLAG_DATA_CONT) {
         int i, s;
         /* cycle through the fragments if they exist */
         for (i = 0, s = size; s > 0; s-= fs->block_size, i++) {
@@ -970,7 +984,7 @@ print_addr_act (FS_INFO *fs, DADDR_T addr, char *buf,
     }
 
     /* this must be an indirect block pointer, so put it in the list */
-    else if (flags & FS_FLAG_META) {
+    else if (flags & FS_FLAG_DATA_META) {
         if (indir_idx < INDIR_SIZ)
             indirl[indir_idx++] = addr;
     }
@@ -985,13 +999,11 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	FFS_INFO *ffs = (FFS_INFO *) fs;
 	FS_INODE *fs_inode;
 	char ls[12];
-    int flags = (FS_FLAG_AONLY | FS_FLAG_CONT | FS_FLAG_META |
-          FS_FLAG_ALLOC | FS_FLAG_UNALLOC );
 
 	fs_inode = ffs_inode_lookup (fs, inum);
 	fprintf(hFile, "inode: %lu\n", (ULONG) inum);
 	fprintf(hFile, "%sAllocated\n", 
-	  (fs_inode->flags&FS_FLAG_ALLOC)?"":"Not ");
+	  (fs_inode->flags&FS_FLAG_META_ALLOC)?"":"Not ");
 
 	fprintf(hFile, "Group: %lu\n", (ULONG)ffs->cg_num);
 
@@ -1039,7 +1051,8 @@ ffs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 	fprintf (hFile, "\nDirect Blocks:\n");
 
 	indir_idx = 0;
-    fs->file_walk(fs, fs_inode, 0, 0, flags, print_addr_act, (char *)hFile);
+    fs->file_walk(fs, fs_inode, 0, 0, FS_FLAG_FILE_AONLY | FS_FLAG_FILE_META, 
+	  print_addr_act, (char *)hFile);
 
     if (printidx != 0)
         fprintf(hFile, "\n");
@@ -1129,15 +1142,13 @@ ffs_open(const char *name, unsigned char ftype)
     fs->root_inum = FFS_ROOTINO;
 	fs->first_inum = FFS_FIRSTINO;
     fs->last_inum = fs->inum_count - 1;
-	fs->niaddr = FFS_NIADDR;
 
 	fs->block_count = gets32(fs, ffs->fs->fs_size);
-    fs->start_block = 0;
+    fs->first_block = 0;
     fs->last_block = fs->block_count - 1;
     fs->block_size = gets32(fs, ffs->fs->fs_fsize);
     fs->file_bsize = gets32(fs, ffs->fs->fs_bsize);
-    fs->addr_bsize = gets32(fs, ffs->fs->fs_bsize);
-    fs->block_frags = gets32(fs, ffs->fs->fs_frag);
+    ffs->block_frags = gets32(fs, ffs->fs->fs_frag);
 	fs->dev_bsize = FFS_DEV_BSIZE;
 
     /*
@@ -1154,6 +1165,7 @@ ffs_open(const char *name, unsigned char ftype)
     fs->dent_walk = ffs_dent_walk;
     fs->file_walk = ffs_file_walk;
 	fs->fsstat = ffs_fsstat;
+	fs->fscheck = ffs_fscheck;
 	fs->istat = ffs_istat;
     fs->close = ffs_close;
 
