@@ -58,10 +58,10 @@
 extern char *tzname[2];
 
 /* ascii version of file types based on types given in fs_dent.h */
-char fs_dent_str[FS_DENT_MAX_STR][2] = {"-","f","c","","d","","b","","r","",
+char fs_dent_str[FS_DENT_MAX_STR][2] = {"-","p","c","","d","","b","","r","",
   "l","","s","h","w"};
 
-char fs_inode_str[FS_INODE_MAX_STR][2] = {"-","f","c","","d","","b","","-","",
+char fs_inode_str[FS_INODE_MAX_STR][2] = {"-","p","c","","d","","b","","-","",
   "l","","s","h","w"};
 
 /* Allocate a fs_dent structure */
@@ -252,7 +252,7 @@ fs_dent_print(FILE *hFile, FS_DENT *fs_dent, int flags, FS_INFO *fs,
 
 
 	/* print a * if it is deleted */
-	if (flags & FS_FLAG_UNALLOC)
+	if (flags & FS_FLAG_NAME_UNALLOC)
 		fprintf(hFile, "* ");
 
 	fprintf(hFile, "%i", fs_dent->inode);
@@ -263,8 +263,8 @@ fs_dent_print(FILE *hFile, FS_DENT *fs_dent, int flags, FS_INFO *fs,
 		  (ULONG)fs_data->id);
 
 	fprintf(hFile, "%s:\t",
-	((fs_inode) && (fs_inode->flags & FS_FLAG_ALLOC) && 
-	  (flags & FS_FLAG_UNALLOC))?"(realloc)":"" );
+	((fs_inode) && (fs_inode->flags & FS_FLAG_META_ALLOC) && 
+	  (flags & FS_FLAG_NAME_UNALLOC))?"(realloc)":"" );
 
 	if (fs_dent->path != NULL) 
 		fprintf(hFile, "%s", fs_dent->path);
@@ -388,9 +388,9 @@ fs_dent_print_mac(FILE *hFile, FS_DENT *fs_dent, int flags,
 
 	/* if filename is deleted add a comment and if the inode is now
 	 * allocated, then add realloc comment */
-	if (flags & FS_FLAG_UNALLOC) 
+	if (flags & FS_FLAG_NAME_UNALLOC) 
 		fprintf(hFile, " (deleted%s)", 
-		  ((fs_inode) && (fs_inode->flags & FS_FLAG_ALLOC))?"-realloc":""); 
+		  ((fs_inode) && (fs_inode->flags & FS_FLAG_META_ALLOC))?"-realloc":""); 
 
 	/* device, inode */ 
 	fprintf(hFile, "|0|%lu", (ULONG)fs_dent->inode);
