@@ -2,7 +2,7 @@
 ** The Sleuth Kit 
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 @stake Inc.  All rights reserved
@@ -11,18 +11,6 @@
 ** Corporation and others. All Rights Reserved.
 */
 
-/* TCT */
-/*++
- * NAME
- *	fstools 3h
- * SUMMARY
- *	file system dumpster diving
- * SYNOPSIS
- *	#include "fstools.h"
- * DESCRIPTION
- * .nf
- */
-
  /*
   * External interface.
   */
@@ -255,18 +243,12 @@ struct FS_INFO {
 	INUM_T	first_inum;			/* first valid inode */
     INUM_T  last_inum;			/* LINUX starts at 1 */
 
-	/* the following is needed by find_inode so that it can
-	 * effeciently allocate enough space */
-	int		niaddr;				/* Number of indirect pointers in inode */
-
 	/* content */
 	DADDR_T block_count;		/* number of blocks */
-	DADDR_T start_block;		/* in case start at 1 */
+	DADDR_T first_block;		/* in case start at 1 */
 	DADDR_T last_block;			/* in case start at 1 */
 	int     block_size;			/* block size in bytes */
 	int     file_bsize;			/* file block size in bytes */
-	int     addr_bsize;			/* indirect block size in bytes */
-	int     block_frags;		/* blocks in logical block */
 	int		dev_bsize;			/* value of DEV_BSIZE constant */
 
 
@@ -274,16 +256,22 @@ struct FS_INFO {
 	unsigned char	flags;		/* flags for image, see below */
 
 
-
 	/* file system specific function pointers */
-    void    (*inode_walk) (FS_INFO *, INUM_T, INUM_T, int, FS_INODE_WALK_FN, char *);
-    void    (*block_walk) (FS_INFO *, DADDR_T, DADDR_T, int, FS_BLOCK_WALK_FN, char *);
+    void    (*block_walk) (FS_INFO *, DADDR_T, DADDR_T, int, 
+	  FS_BLOCK_WALK_FN, char *);
+
+    void    (*inode_walk) (FS_INFO *, INUM_T, INUM_T, int, 
+	  FS_INODE_WALK_FN, char *);
     FS_INODE *(*inode_lookup) (FS_INFO *, INUM_T);
-    void    (*dent_walk) (FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
-    void    (*file_walk) (FS_INFO *, FS_INODE *, u_int32_t, u_int16_t, int, 
-	  FS_FILE_WALK_FN, char *);
-	void	(*fsstat) (FS_INFO *, FILE *);
 	void	(*istat) (FS_INFO *, FILE *, INUM_T, int, int32_t);
+    void    (*file_walk) (FS_INFO *, FS_INODE *, u_int32_t, u_int16_t, 
+	  int, FS_FILE_WALK_FN, char *);
+
+    void    (*dent_walk) (FS_INFO *, INUM_T, int, FS_DENT_WALK_FN, char *);
+
+	void	(*fsstat) (FS_INFO *, FILE *);
+	void	(*fscheck) (FS_INFO *, FILE *);
+
     void    (*close) (FS_INFO *);
 };
 
@@ -293,23 +281,41 @@ struct FS_INFO {
 #define FS_HAVE_DTIME	0x04
 #define FS_HAVE_SEQ		0x08
 
-/* values for flags passed to the walk functions */
-#define FS_FLAG_LINK	(1<<0)		/* link count > 0 */
-#define FS_FLAG_UNLINK	(1<<1)		/* link count == 0 */
-#define FS_FLAG_ALLOC	(1<<2)		/* allocated */
-#define FS_FLAG_UNALLOC	(1<<3)		/* unallocated */
-#define FS_FLAG_USED	(1<<4)		/* used */
-#define FS_FLAG_UNUSED	(1<<5)		/* never used */
-#define FS_FLAG_META    (1<<6)   	/* meta data block */
-#define FS_FLAG_ALIGN	(1<<7)		/* block align (i.e. send a whole block) */
-#define FS_FLAG_TMHDR	(1<<8)		/* show tm header */
+//#define FS_FLAG_TMHDR	(1<<9)		/* show tm header */
+
+/* flags that are used for dent_walk and FS_NAME */
+#define FS_FLAG_NAME_ALLOC	 (1<<0)		/* allocated */
+#define FS_FLAG_NAME_UNALLOC (1<<1)		/* unallocated */
+#define FS_FLAG_NAME_RECURSE (1<<2)		/* recurse on directories */
+
+/* flags that are used for inode_walk and FS_INODE */
+#define FS_FLAG_META_LINK	(1<<0)		/* link count > 0 */
+#define FS_FLAG_META_UNLINK	(1<<1)		/* link count == 0 */
+#define FS_FLAG_META_ALLOC	(1<<2)		/* allocated */
+#define FS_FLAG_META_UNALLOC (1<<3)		/* unallocated */
+#define FS_FLAG_META_USED	(1<<4)		/* used */
+#define FS_FLAG_META_UNUSED	(1<<5)		/* never used */
+
+/* flags that are used for block_walk and any data units 
+ * Including the data units in the action of file_walk */
+#define FS_FLAG_DATA_ALLOC	(1<<0)	/* allocated */
+#define FS_FLAG_DATA_UNALLOC (1<<1)	/* unallocated */
+#define FS_FLAG_DATA_CONT	(1<<2)	/* allocated for file content */
+#define FS_FLAG_DATA_META	(1<<3)	/* allocated for meta data */
+#define FS_FLAG_DATA_BAD	(1<<4)	/* marked as bad by the FS */
+#define FS_FLAG_DATA_ALIGN	(1<<5)	/* block align (i.e. send a whole block) */
+
+
+
+/* Flags used when calling file_walk, action of file_walk uses
+ * the FS_FLAG_DATA_ flags */
+#define FS_FLAG_FILE_AONLY	(1<<0)		/* only copy address to callback */
+#define FS_FLAG_FILE_SLACK	(1<<1)		/* return slack space too */
+#define FS_FLAG_FILE_RECOVER (1<<2)		/* Recover a deleted file */
+#define FS_FLAG_FILE_META	(1<<3)		/* return meta data units too */
+#define FS_FLAG_FILE_NOSPARSE (1<<4)	/* don't return sparse data units */
+#define FS_FLAG_FILE_NOABORT (1<<5)		/* do not abort when errors are found*/
 
-#define FS_FLAG_RECURSE (1<<9)		/* recurse on directories in dent_walk */
-#define FS_FLAG_CONT	(1<<10)		/* call file_walk on content blocks */
-#define FS_FLAG_AONLY	(1<<11)		/* on file_walk, only copy address */
-#define FS_FLAG_SLACK	(1<<12)		/* file_walk: return slack space too */
-#define FS_FLAG_NOABORT (1<<13) 	/* when doing a walk, do not abort for
-									 * invalid data - used in ifind */
 
 /***************************************************************
  * I/O buffer, used for all forms of I/O.
@@ -405,7 +411,7 @@ struct FS_INODE {
 	FS_NAME *name;
 	char	*link;				/* used if this is a symbolic link */
 
-	int 	flags;			/* flags for inode (allocated etc.) */
+	int 	flags;			/* flags FS_FLAG_META_* */
 };
 
 extern FS_INODE *fs_inode_alloc(int, int);
