@@ -251,7 +251,7 @@ find_inode_path (FS_INFO *fs, FS_DENT *fs_dent, int flags, char *ptr)
 	if ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR) {
 
 		fs->dent_walk(fs, fs_dent->inode,
-		  FS_FLAG_ALLOC | FS_FLAG_UNALLOC,
+		  FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC,
 		  find_inode_path, (char *) 0);
 	}
 
@@ -265,7 +265,7 @@ find_inode_path (FS_INFO *fs, FS_DENT *fs_dent, int flags, char *ptr)
 }
 
 /*
- * file walk action for non-ntfs
+ * file_walk action for non-ntfs
  */
 static u_int8_t
 find_inode_file_act(FS_INFO *fs, DADDR_T addr, char *buf, 
@@ -294,7 +294,7 @@ find_inode_file_act(FS_INFO *fs, DADDR_T addr, char *buf,
 
 
 /* 
- * file walk action callback for ntfs  
+ * file_walk action callback for ntfs  
  *
  */
 static u_int8_t
@@ -318,13 +318,16 @@ find_inode_ntfs_file(FS_INFO *fs, DADDR_T addr, char *buf,
 /*
 ** find_inode
 **
-** Main function that calls the direct and indirect calls
+** Callback action for inode_walk
 */
 static u_int8_t
 find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr) 
 {
-	int myflags = (FS_FLAG_AONLY | FS_FLAG_CONT | FS_FLAG_META | 
-	  FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_NOABORT);
+	int file_flags = (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_NOABORT);
+
+	/* If the meta data structure is unallocated, then set the recovery flag */
+	if (flags & FS_FLAG_META_UNALLOC)
+		file_flags |= FS_FLAG_FILE_RECOVER;
 
 	curinode = inum;
 
@@ -332,12 +335,12 @@ find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr)
 	if ((fs->ftype & FSMASK) == NTFS_TYPE) {
 		FS_DATA *data = fsi->attr;
 
-		myflags |= FS_FLAG_SLACK;
+		file_flags |= FS_FLAG_FILE_SLACK;
 		while ((data) && (data->flags & FS_DATA_INUSE)) {
 			curtype = data->type;
 			curid = data->id;
 			if (data->flags & FS_DATA_NONRES) {
-				fs->file_walk(fs, fsi, data->type, data->id, myflags, 
+				fs->file_walk(fs, fsi, data->type, data->id, file_flags, 
 				  find_inode_ntfs_file, ptr);
 			}
 			data = data->next;
@@ -345,8 +348,8 @@ find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr)
 		return WALK_CONT;
 	}
 	else if ((fs->ftype & FSMASK) == FATFS_TYPE) {
-		myflags |= FS_FLAG_SLACK;
-		fs->file_walk(fs, fsi, 0, 0, myflags, 
+		file_flags |= FS_FLAG_FILE_SLACK;
+		fs->file_walk(fs, fsi, 0, 0, file_flags, 
 		  find_inode_file_act, ptr);
 	}
 	/* UNIX do not need the SLACK flag because they use fragments - if the
@@ -354,7 +357,7 @@ find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr)
 	 * correlated with the incorrect inode
 	 */
 	else {
-		fs->file_walk(fs, fsi, 0, 0, myflags, 
+		fs->file_walk(fs, fsi, 0, 0, file_flags, 
 		  find_inode_file_act, ptr);
 	}
 
@@ -369,7 +372,7 @@ find_inode(FS_INFO *fs, INUM_T inum, FS_INODE *fsi, int flags, char *ptr)
 static u_int8_t
 block_act (FS_INFO *fs, DADDR_T addr, char *buf, int flags, char *ptr)
 {
-	if (flags & FS_FLAG_META)
+	if (flags & FS_FLAG_DATA_META)
 		printf("Meta Data\n");
 	else
 		printf("Inode not found\n");
@@ -457,7 +460,7 @@ main(int argc, char **argv)
 			fprintf(stderr, "Looking for %s\n", cur_dir);
 
 		fs->dent_walk(fs, fs->root_inum,
-	  	  FS_FLAG_ALLOC | FS_FLAG_UNALLOC,
+	  	  FS_FLAG_NAME_ALLOC | FS_FLAG_NAME_UNALLOC,
 		  find_inode_path, (char *) 0);
 
 		if (0 == (localflags & FOUND_ONE)) {
@@ -499,8 +502,9 @@ main(int argc, char **argv)
 		}
 
 		fs->inode_walk(fs, fs->first_inum, fs->last_inum,
-	  	  FS_FLAG_LINK | FS_FLAG_UNLINK | 
-	  	  FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_USED | FS_FLAG_UNUSED,
+	  	  FS_FLAG_META_LINK | FS_FLAG_META_UNLINK | 
+	  	  FS_FLAG_META_ALLOC | FS_FLAG_META_UNALLOC | 
+		  FS_FLAG_META_USED | FS_FLAG_META_UNUSED,
 		  find_inode, (char *) 0);
 
 		/* 
@@ -509,7 +513,8 @@ main(int argc, char **argv)
 		 * a meta data block */
 		if (0 == (localflags & FOUND_ONE)) {
 			fs->block_walk(fs, block, block,
-				FS_FLAG_UNALLOC | FS_FLAG_ALLOC | FS_FLAG_META | FS_FLAG_AONLY, 
+				FS_FLAG_DATA_UNALLOC | FS_FLAG_DATA_ALLOC | 
+				FS_FLAG_DATA_META | FS_FLAG_DATA_CONT,
 				block_act, (char *) 0);
 		}
 	}
