@@ -5,7 +5,7 @@
 ** Content and meta data layer support for the NTFS file system
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -520,11 +520,13 @@ ntfs_make_data_run(NTFS_INFO *ntfs, ntfs_runlist *runlist,
  * This gives us an interface to call an action on data and not care if
  * it is resident or not.
  *
- * used flag values: FS_FLAG_AONLY, FS_FLAG_SLACK, FS_FLAG_ALLOC, FS_FLAG_UNALLOC
- * unused flag values: FS_FLAG_META, FS_FLAG_CONT, 
+ * used flag values: FS_FLAG_FILE_AONLY, FS_FLAG_FILE_SLACK, 
+ * FS_FLAG_FILE_NOSPARSE, FS_FLAG_FILE_NOAOBRT
  *
- * FS_FLAG_UNALLOC in this case means sparse entries (non-resident only)
+ * Action uses: FS_FLAG_DATA_CONT
+ * @@@ Not implemented FS_FLAG_DATA_ALLOC _UNALLOC yet
  *
+ * No notion of META
  */
 static void
 ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data, 
@@ -534,24 +536,16 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 	int myflags;
 
 	/* Process the resident buffer 
-	 *
-	 * Resident has no concept of UNALLOCATED or SLACK
 	 */
 	if (fs_data->flags & FS_DATA_RES)  {
 		char *buf = NULL;
 
-		/* if it is resident, we only care about it if the content is
-		 * wanted as there are no address or meta data about it
-		 */
-		if ((flags & FS_FLAG_CONT) == 0)
-			return;
-
-		if ((flags & FS_FLAG_AONLY) == 0) {
+		if ((flags & FS_FLAG_FILE_AONLY) == 0) {
 			buf = mymalloc(fs_data->size);
 			memcpy(buf, fs_data->buf, fs_data->size);
 		}
 
-		myflags = FS_FLAG_CONT;
+		myflags = FS_FLAG_DATA_CONT;
 		action(fs, ntfs->root_mft_addr, buf, fs_data->size, myflags, ptr);
 			
 		if (buf)
@@ -567,12 +561,12 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 		FS_DATA_RUN *fs_data_run;
 
 		/* if we want the slack space too, then use the runlen  */
-		if (flags & FS_FLAG_SLACK)
+		if (flags & FS_FLAG_FILE_SLACK)
 			fsize = fs_data->runlen;
 		else 
 			fsize = fs_data->size;
 
-		if ((flags & FS_FLAG_AONLY) == 0)  {
+		if ((flags & FS_FLAG_FILE_AONLY) == 0)  {
 			fs_buf = fs_buf_alloc(fs->block_size);	
 			buf = fs_buf->data;
 		}
@@ -597,7 +591,7 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 				/* If the address is too large then give an error unless
 				 * the no abort flag is set */
 				if (addr > fs->last_block) {
-					if (flags & FS_FLAG_NOABORT) {
+					if (flags & FS_FLAG_FILE_NOABORT) {
 						if (verbose) {
 							fprintf (logfp,
 							  "Invalid address in run (too large): %lu",
@@ -611,7 +605,7 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 					}
 				}
 
-				if ((flags & FS_FLAG_AONLY) == 0) {
+				if ((flags & FS_FLAG_FILE_AONLY) == 0) {
 					/* sparse files just get 0s */
 					if (fs_data_run->flags & FS_DATA_SPARSE)
 						memset (buf, 0, fs->block_size);
@@ -626,21 +620,21 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 				else
 					bufsize = (int)fsize;
 
-				myflags = FS_FLAG_CONT;
+				myflags = FS_FLAG_DATA_CONT;
 
-				/* Only do sparse clusters if UNALLOC is set */
+				/* Only do sparse clusters if NOSPARSE is not set */
 				if ((fs_data_run->flags & FS_DATA_SPARSE) && 
-				  (flags & FS_FLAG_UNALLOC)) {
-					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags | FS_FLAG_UNALLOC, ptr)) {
-						if ((flags & FS_FLAG_AONLY) == 0)
+				  (0 == (flags & FS_FLAG_FILE_NOSPARSE))) {
+					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags, ptr)) {
+						if ((flags & FS_FLAG_FILE_AONLY) == 0)
 							fs_buf_free(fs_buf);
 						return;
 					}
 				}
-				else if (((fs_data_run->flags & FS_DATA_SPARSE) == 0) && 
-				  (flags & FS_FLAG_ALLOC)) {
-					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags | FS_FLAG_ALLOC, ptr)) {
-						if ((flags & FS_FLAG_AONLY) == 0)
+
+				else if ((fs_data_run->flags & FS_DATA_SPARSE) == 0) {
+					if (WALK_STOP == action(fs, addr, buf, bufsize, myflags, ptr)) {
+						if ((flags & FS_FLAG_FILE_AONLY) == 0)
 							fs_buf_free(fs_buf);
 						return;
 					}
@@ -662,7 +656,7 @@ ntfs_data_walk(NTFS_INFO *ntfs, FS_DATA *fs_data,
 			fs_data_run = fs_data_run->next;
 		}
 
-		if ((flags & FS_FLAG_AONLY) == 0) 
+		if ((flags & FS_FLAG_FILE_AONLY) == 0) 
 			fs_buf_free(fs_buf);
 
 	} /* end of non-res */
@@ -1016,8 +1010,7 @@ ntfs_proc_attrlist(NTFS_INFO *ntfs, FS_INODE *fs_inode,
 	act_buf = buf = mymalloc (act_bufleft);
 	endaddr = (DADDR_T)buf + fs_data_attrlist->size;
 
-	ntfs_data_walk(ntfs, fs_data_attrlist, 
-	  (FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_CONT), 
+	ntfs_data_walk(ntfs, fs_data_attrlist, 0,
 	  buffer_action, "attrlist");
 
 	/* this value should be zero, if not then we didn't read all of the
@@ -1163,13 +1156,13 @@ ntfs_copy_inode(NTFS_INFO *ntfs, FS_INODE *fs_inode)
 
 	/* add the flags */
 	fs_inode->flags = ((getu16(fs, ntfs->mft->flags) & NTFS_MFT_INUSE) ?
-	  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+	  FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
 	fs_inode->flags |= (getu16(fs, ntfs->mft->link) ? 
-	  FS_FLAG_LINK : FS_FLAG_UNLINK);
+	  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 
 	/* MFT entries are only allocated when needed, so it has been used */
-	fs_inode->flags |= FS_FLAG_USED;
+	fs_inode->flags |= FS_FLAG_META_USED;
 
 	/* Process the attribute sequence to fill in the fs_inode->attr
 	 * list and the other info such as size and times
@@ -1250,8 +1243,7 @@ ntfs_load_attrdef (NTFS_INFO *ntfs)
 	act_buf = mymalloc(act_bufleft);
     ntfs->attrdef = (ntfs_attrdef *)act_buf;
     
-    ntfs_data_walk(ntfs, fs_data,
-      FS_FLAG_ALLOC | FS_FLAG_UNALLOC | FS_FLAG_CONT, buffer_action, "");
+    ntfs_data_walk(ntfs, fs_data, 0, buffer_action, "");
 
     if (act_bufleft)
         error ("space still left after walking $Attr data");
@@ -1368,8 +1360,14 @@ ntfs_load_ver (NTFS_INFO *ntfs)
 
 /*
  *
- * flag values: FS_FLAG_{UN}ALLOC, FS_FLAG_META, FS_FLAG_CONT, FS_FLAG_AONLY
- * FS_FLAG_SLACK
+ * flag values: FS_FLAG_FILE_AONLY, FS_FLAG_FILE_SLACK, FS_FLAG_FILE_NOSPARSE
+ *  FS_FLAG_FILE_NOABORT
+ * 
+ * nothing special is done for FS_FLAG_FILE_RECOVER
+ *
+ * action uses: FS_FLAG_DATA_CONT
+ *
+ * No notion of meta with NTFS
  *
  * a type of 0 will use $Data for files and IDXROOT for directories
  * an id of 0 will ignore the id and just find the first entry with the type
@@ -1385,12 +1383,6 @@ ntfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 	if (fs_inode->attr == NULL)
 		return;
 
-	/* there are no "meta" blocks here so return if they don't want
-	 * content  
-	 */
-	if ((flags & FS_FLAG_CONT) == 0)
-		return;
-
 	/* if no type was given, then use DATA for files and IDXROOT for dirs */
 	if (type == 0) {
 		if ((fs_inode->mode & FS_INODE_FMT) == FS_INODE_DIR)
@@ -1417,7 +1409,7 @@ ntfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 
 
 /*
- * flags: FS_FLAG_ALLOC and FS_FLAG_UNALLOC
+ * flags: FS_FLAG_DATA_ALLOC and FS_FLAG_UNALLOC
  *
  * @@@ We should probably consider some data META, but it is tough with
  * the NTFS design ...
@@ -1437,9 +1429,9 @@ ntfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     /*
      * Sanity checks.
      */
-	if (start < fs->start_block || start > fs->last_block)
+	if (start < fs->first_block || start > fs->last_block)
 		error("%s: invalid start block number: %lu", myname, (ULONG) start);
-	if (last < fs->start_block || last > fs->last_block)
+	if (last < fs->first_block || last > fs->last_block)
 		error("%s: invalid last block number: %lu", myname, (ULONG) last);
 
 	bits_p_clust = 8 * fs->block_size;
@@ -1466,7 +1458,7 @@ ntfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 		/* identify if the cluster is allocated or not */
 		b = addr % bits_p_clust;
 		myflags = (((bmap->data[b/8]) & (0x1 << (b % 8)) ) ?
-		  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+		  FS_FLAG_DATA_ALLOC : FS_FLAG_DATA_UNALLOC);
 
 		if (flags & myflags) {
 			fs_read_block(fs, fs_buf, fs->block_size, addr, "data block");
@@ -1487,9 +1479,12 @@ ntfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
 
 
 /*
- * Unallocated inodes are determined using the INUSE flag and never
- * the link value because unused records have a link > 0
+ * inode_walk
  *
+ * Flags: FS_FLAG_META_ALLOC, FS_FLAG_META_UNALLOC, FS_FLAG_META_LINK,
+ * FS_FLAG_META_UNLINK, FS_FLAG_META_USED
+ *
+ * Not used: FS_FLAG_META_UNUSED (Only allocated when needed)
  */
 void
 ntfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
@@ -1531,15 +1526,15 @@ ntfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 
 		/* check flags */
 		myflags = ((getu16(fs, ntfs->mft->flags) & NTFS_MFT_INUSE) ? 
-		  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+		  FS_FLAG_META_ALLOC : FS_FLAG_META_UNALLOC);
 
 		myflags |= (getu16(fs, ntfs->mft->link) ? 
-		  FS_FLAG_LINK : FS_FLAG_UNLINK);
+		  FS_FLAG_META_LINK : FS_FLAG_META_UNLINK);
 
 		/* MFT entries are only allocated when needed, so it must have 
 		 * been used
 		 */
-		myflags |= FS_FLAG_USED;
+		myflags |= FS_FLAG_META_USED;
 
 		if ((flags & myflags) != myflags)
 			continue;
@@ -1560,6 +1555,11 @@ ntfs_inode_walk(FS_INFO *fs, INUM_T start, INUM_T last, int flags,
 }
 
 
+static void
+ntfs_fscheck(FS_INFO *fs, FILE *hFile)
+{
+	error ("fscheck not implemented for NTFS yet");
+}
 
 
 static void
@@ -1621,7 +1621,7 @@ ntfs_fsstat(FS_INFO *fs, FILE *hFile)
     fprintf(hFile, "Cluster Size: %d\n", ntfs->csize_b);
 
     fprintf(hFile, "Total Cluster Range: %lu - %lu\n", 
-	  (ULONG)fs->start_block,
+	  (ULONG)fs->first_block,
       (ULONG)fs->last_block);
 
 	
@@ -1674,14 +1674,12 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
 {
     FS_INODE *fs_inode;
 	FS_DATA *fs_data;
-    int flags = (FS_FLAG_CONT | FS_FLAG_AONLY | FS_FLAG_SLACK |
-      FS_FLAG_ALLOC | FS_FLAG_UNALLOC);
 
     fs_inode = ntfs_inode_lookup (fs, inum);
     fprintf(hFile, "MFT Entry: %lu\n", (ULONG) inum);
 	fprintf(hFile, "Sequence: %lu\n", (ULONG) fs_inode->seq);
     fprintf(hFile, "%sAllocated\n",
-      (fs_inode->flags&FS_FLAG_ALLOC)?"":"Not ");
+      (fs_inode->flags & FS_FLAG_META_ALLOC)? "" : "Not ");
 
 	fprintf(hFile, "UID: %d\n", (int)fs_inode->uid);
 
@@ -1797,7 +1795,8 @@ ntfs_istat (FS_INFO *fs, FILE *hFile, INUM_T inum, int numblock,
         if (fs_data->flags & FS_DATA_NONRES) {
             printidx = 0;
 
-            fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, flags,
+            fs->file_walk(fs, fs_inode, fs_data->type, fs_data->id, 
+			   (FS_FLAG_FILE_AONLY | FS_FLAG_FILE_SLACK),
                print_addr_act, (char *)hFile);
             if (printidx != 0)
                 printf("\n");
@@ -1861,7 +1860,7 @@ ntfs_open(const char *name, unsigned char ftype)
 	
 	ntfs->ssize_b = getu16(fs, ntfs->fs->ssize);
 	ntfs->csize_b = ntfs->fs->csize * ntfs->ssize_b;
-    fs->start_block = 0;
+    fs->first_block = 0;
 
 	/* This field is defined as 64-bits but according to the
 	 * NTFS drivers in Linux, windows only uses 32-bits
@@ -1870,8 +1869,6 @@ ntfs_open(const char *name, unsigned char ftype)
     fs->last_block = fs->block_count - 1;
     fs->block_size = ntfs->csize_b;
     fs->file_bsize = fs->block_size;
-    fs->addr_bsize = 0;
-    fs->block_frags = 1;
     fs->dev_bsize = NTFS_DEV_BSIZE;
 
 
@@ -1900,6 +1897,7 @@ ntfs_open(const char *name, unsigned char ftype)
     fs->inode_lookup = ntfs_inode_lookup;
     fs->dent_walk = ntfs_dent_walk;
 	fs->fsstat = ntfs_fsstat;
+	fs->fscheck = ntfs_fscheck;
 	fs->istat = ntfs_istat;
     fs->close = ntfs_close;
 
