@@ -5,7 +5,7 @@
 ** name layer support for the NTFS file system
 **
 ** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003 Brian Carrier.  All rights reserved
+** Copyright (c) 2003-2004 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 Brian Carrier, @stake Inc.  All rights reserved
@@ -242,10 +242,10 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 				free(fs_dent->fsi);
 				fs_dent->fsi = NULL;
 			}
-			myflags = FS_FLAG_UNALLOC;
+			myflags = FS_FLAG_NAME_UNALLOC;
 		}
 		else
-			myflags = FS_FLAG_ALLOC;
+			myflags = FS_FLAG_NAME_ALLOC;
 
 
 		if ((flags & myflags) == myflags) {
@@ -255,8 +255,8 @@ ntfs_dent_idxentry(NTFS_INFO *ntfs, ntfs_idxentry *idxe,
 		}
 
 		/* Recurse if we need to */
-		if ((myflags & FS_FLAG_ALLOC) &&
-		   (flags & FS_FLAG_RECURSE) &&
+		if ((myflags & FS_FLAG_NAME_ALLOC) &&
+		   (flags & FS_FLAG_NAME_RECURSE) &&
 		   (!ISDOT(fs_dent->name)) &&
 		   ((fs_dent->fsi->mode & FS_INODE_FMT) == FS_INODE_DIR) &&
 		   (fs_dent->inode)) {
@@ -415,10 +415,10 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 	 * So, we'll make 'em up by making a FS_DENT structure for
 	 * a '.' and '..' entry and call the action
 	 */
-	if ((inum != fs->root_inum) && (flags & FS_FLAG_ALLOC)) {
+	if ((inum != fs->root_inum) && (flags & FS_FLAG_NAME_ALLOC)) {
 		FS_DENT *fs_dent = fs_dent_alloc(16);
 		FS_NAME *fs_name;
-		int myflags = FS_FLAG_ALLOC;
+		int myflags = FS_FLAG_NAME_ALLOC;
 
 		if (verbose)
 			fprintf (logfp, "ntfs_dent_walk: Creating . and .. entries\n");
@@ -536,7 +536,7 @@ ntfs_dent_walk(FS_INFO *fs, INUM_T inum, int flags,
 		fprintf (logfp, "ntfs_dent_walk: Copying $IDX_ALLOC into buffer\n");
 
 	fs->file_walk(fs, fs_inode, NTFS_ATYPE_IDXALLOC, 0, 
-		FS_FLAG_CONT | FS_FLAG_SLACK | FS_FLAG_ALLOC | FS_FLAG_UNALLOC,
+		FS_FLAG_FILE_SLACK,
 		idxalloc_action, "index alloc");
 
 	if (bufleft > 0)
@@ -813,7 +813,7 @@ ntfs_find_file (FS_INFO *fs, INUM_T inode_toid, u_int32_t type_toid,
 	fs_dent->inode = inode_toid;
     fs_dent->fsi = fs_inode = fs->inode_lookup(fs, inode_toid);
 	myflags = ((getu16(fs, ntfs->mft->flags) & NTFS_MFT_INUSE) ?
-		  FS_FLAG_ALLOC : FS_FLAG_UNALLOC);
+		  FS_FLAG_NAME_ALLOC : FS_FLAG_NAME_UNALLOC);
 
 	/* Get the name for the attribute - if specified */
 	if (type_toid != 0) {
