@@ -60,22 +60,21 @@ rawfs_block_walk(FS_INFO *fs, DADDR_T start, DADDR_T last, int flags,
     /*
      * Sanity checks.
      */
-    if (start < fs->start_block || start > fs->last_block)
+    if (start < fs->first_block || start > fs->last_block)
 		error("%s: invalid start block number: %lu", myname, (ULONG) start);
 
-    if (last < fs->start_block || last > fs->last_block || last < start)
+    if (last < fs->first_block || last > fs->last_block || last < start)
 		error("%s: invalid last block number: %lu", myname, (ULONG) last);
 
-	if (!(flags & FS_FLAG_ALLOC))
+	if (!(flags & FS_FLAG_DATA_ALLOC))
 		return;
 
-    for (addr = start; addr <= last; addr += fs->block_frags) {
-
+    for (addr = start; addr <= last; addr ++) {
 		fs_read_block(fs, fs_buf, fs->block_size, addr, 
 		  "block_walk: data block");
 
 		if (WALK_STOP == action(fs, addr, fs_buf->data,
-		  FS_FLAG_ALLOC, ptr)) {
+		  FS_FLAG_DATA_ALLOC | FS_FLAG_DATA_CONT, ptr)) {
     		fs_buf_free(fs_buf);
 			return;
 		}
@@ -170,7 +169,6 @@ rawfs_open(const char *name, unsigned char ftype)
     fs->root_inum = 0;
 	fs->first_inum = 0;
     fs->last_inum = 0;
-	fs->niaddr = 0;
 
 	len = LSEEK(fs->fd, 0, SEEK_END);
 	if (len < 0)
@@ -180,12 +178,10 @@ rawfs_open(const char *name, unsigned char ftype)
 	if (len % 512) 
 		fs->block_count++;
 
-    fs->start_block = 0;
+    fs->first_block = 0;
     fs->last_block = fs->block_count - 1;
     fs->block_size = 512;
     fs->file_bsize = 512;
-    fs->addr_bsize = 512;
-    fs->block_frags = 1;
 	fs->dev_bsize = 512;
     fs->seek_pos = -1;
 
